// ComparisonTest.java
//
// ICS 22 / CSE 22 Fall 2012
// Code Example
//
// Here is an example of a generic "find the minimum" algorithm.

import java.util.ArrayList;


public class ComparisonTest
{
	public static void main(String[] args)
	{
		CalendarDate d1 = new CalendarDate(2003, 1, 22);
		CalendarDate d2 = new CalendarDate(2006, 1, 19);
	
		// We expect this to be a negative number
		System.out.println("This should be a negative number: " + d1.compareTo(d2));

		// The reason why we're saying ArrayList<Comparable>,
		// rather than ArrayList<CalendarDate>, is a somewhat technical
		// one.  The findMin() method that we'll be using to find
		// the minimum date expects an ArrayList<Comparable> as a
		// parameter; if we try to pass it an ArrayList<CalendarDate>
		// instead, we'll get an error.  (There are some creative
		// ways around this, but they're a bit more in-depth than
		// I'd like to take us for now.)
		ArrayList<Comparable> dates = new ArrayList<Comparable>();
		dates.add(new CalendarDate(2005, 7, 19));
		dates.add(new CalendarDate(2005, 11, 1));
		dates.add(new CalendarDate(1988, 7, 10));
		dates.add(new CalendarDate(1988, 7, 25));
		dates.add(new CalendarDate(1987, 8, 13));
		dates.add(new CalendarDate(2006, 1, 17));
		dates.add(new CalendarDate(1974, 1, 1));
		dates.add(new CalendarDate(1941, 9, 19));
		dates.add(new CalendarDate(1988, 3, 14));
		System.out.println("Minimum date is " + findMin(dates));
	}


	// findMin() takes an ArrayList of objects that are
	// Comparable and finds the smallest object in the list.
	// Note that this algorithm is indifferent about what kind
	// of objects are contained within the list.  If we had
	// ten classes that implemented Comparable, this method
	// would work for all ten; if we then added ten more classes
	// that implement Comparable, the method would then work for
	// all twenty, with no changes required.
	//
	// The key to why this method works is because the call to
	// compareTo() will be "polymorphic," like all calls to methods
	// in Java.  Automatically, the right version of the method
	// will be called (based on the type of the objects at run-time).
	// In our example, the ArrayList really contains CalendarDates, so
	// it's CalendarDate's version of compareTo() that's getting called
	// each time.
	private static Comparable findMin(ArrayList<Comparable> a)
	{
		// If there are no objects being considered (either because a is
		// null -- meaning that there is no list -- or a is a list with
		// no elements), we'll return null, as a way of saying that
		// there is no minimum element.

		if (a == null || a.size() == 0)
		{
			return null;
		}

		// From here, we do a typical "find the minimum" algorithm:
		//
		//     let min be the first element of the list
		//     for each element e in the list other than the first
		//     {
		//         if e < min
		//             min = e
		//     }
		//     return min

		Comparable min = a.get(0);
		
		for (int i = 1; i < a.size(); i++)
		{
			Comparable other = a.get(i);
			
			if (other.compareTo(min) < 0)
			{
				min = other;
			}
		}
		
		return min;
	}
}
